@@ -205,20 +205,24 @@ def main():
         )
     elif r.status == OptimizationResponseStatus.SUCCESS:
         output = args.output
-        original_image_abs_path = os.path.abspath(os.path.dirname(image_path))
+        optimized_image_dir_abs_path = os.path.abspath(
+            os.path.dirname(image_path)
+        )
+        base_filename, file_extension = os.path.splitext(image_path)
+        output_filename = '{}{}'.format(
+            base_filename,
+            constants.PrecursorToExtensionEnum[precursor.name].value
+        )
         if output:
+            if os.path.isdir(output):
+                output = os.path.join(output, output_filename)
             # Let's assume just the filename was set as output, and we then
             # store it where the original image is.
-            if not os.path.isabs(output):
-                output = os.path.join(original_image_abs_path, output)
+            else:
+                output = os.path.join(optimized_image_dir_abs_path, output)
         else:
-            base_filename, file_extension = os.path.splitext(image_path)
-            output_filename = '{}{}'.format(
-                base_filename,
-                constants.PrecursorToExtensionEnum[precursor.name].value
-            )
             output = os.path.join(
-                original_image_abs_path,
+                optimized_image_dir_abs_path,
                 output_filename
             )
         loop.run_until_complete(
