@@ -13,6 +13,8 @@ from aiohttp_retry import (
     RetryClient,
     RetryOptions,
 )
+import ssl
+import certifi
 
 from .api_config import (
     ApiEndpoint,
@@ -22,7 +24,7 @@ from .stream_helpers import (
     stream_from_argument,
     ImageArg,
 )
-from .constants import PrecursorEnum
+from . import constants
 
 # How many times to retry a request.
 RETRY_COUNT = 5
@@ -56,6 +58,7 @@ class UseTag(Enum):
     INITIAL_LOOKUP = "InitialLookup"
     POST_BYTES = "PostBytes"
     CHECK_PROCESSING_STATUS = "CheckProcessingStatus"
+    DOWNLOAD_FROM_PRESIGNED_URL = 'DownloadFromPresignedURL'
 
 
 class EnvelopeOptions(NamedTuple):
@@ -72,7 +75,7 @@ class EnvelopeOptions(NamedTuple):
 
 class ApiRequest(NamedTuple):
     one_pass_info: OnePassInfo
-    precursor_name: PrecursorEnum
+    precursor_name: constants.PrecursorEnum
     adjustments: Dict
     envelope_options: Optional[EnvelopeOptions] = None
 
@@ -107,6 +110,9 @@ class ToilmoreHTTPRequester(object):
 
         self._use_host = self._get_api_host()
         self._logger = logging.getLogger('toilmoresdk')
+        self._sslcontext = ssl.create_default_context(
+            cafile=certifi.where()
+        )
 
     def _debug_message(self, use_tag: UseTag, msg: str) -> None:
 
@@ -131,54 +137,11 @@ class ToilmoreHTTPRequester(object):
         :return:
         """
         file_hash = api_request.one_pass_info.file_hash
-        use_envelope_options = api_request.envelope_options
-        force_reprocessing = (
-            use_envelope_options.force_reprocessing
-            if use_envelope_options else False
+        request_json_body = self._get_request_json_body(
+            api_request,
+            file_hash,
+            attempt_no
         )
-        adjustments = api_request.adjustments
-        original_image_size_in_bytes = api_request.one_pass_info.file_size
-
-        if not force_reprocessing:
-            force_reprocessing = False
-        if not adjustments:
-            adjustments = {}
-
-        expected_optimized_image_filename = (
-            use_envelope_options.expected_optimized_image_filename
-            if use_envelope_options else None
-        )
-        if not expected_optimized_image_filename:
-            expected_optimized_image_filename = (
-                "/python-sdk-call/at-hash/optimized/" + file_hash
-            )
-
-        # Let's use a codified relative path so that we can test-match in tests.
-        use_relative_path = (
-            "/python-sdk-call/at-hash/input/" + file_hash + "/attempt_" +
-            str(attempt_no)
-        )
-
-        request_json_body = {
-            "image_file_hash": file_hash,
-            "original_image_size_in_bytes": original_image_size_in_bytes,
-            "relative_path": use_relative_path,
-            "expected_optimized_image_filename":
-                expected_optimized_image_filename,
-            "precursor_name": api_request.precursor_name.value,
-            "domains": [
-                self._domain
-            ],
-            "force_reprocessing": force_reprocessing,
-            "shimmercat_job_record": {
-                "adjustments": adjustments,
-            }
-        }
-        max_overhead = (
-            use_envelope_options.max_overhead if use_envelope_options else None
-        )
-        if max_overhead:
-            request_json_body["max_overhead"] = max_overhead
         retry_options = RetryOptions(
             attempts=attempts,
             start_timeout=20,
@@ -197,6 +160,7 @@ class ToilmoreHTTPRequester(object):
             headers={
                 'Authorization': self._make_auth_token()
             },
+            ssl=self._sslcontext
         ) as response:
             status_code = response.status
             if int(status_code / 100) == 5:
@@ -240,55 +204,13 @@ class ToilmoreHTTPRequester(object):
         :param attempts:
         :return:
         """
+
         file_hash = api_request.one_pass_info.file_hash
-        use_envelope_options = api_request.envelope_options
-        force_reprocessing = (
-            use_envelope_options.force_reprocessing
-            if use_envelope_options else False
+        request_json_body = self._get_request_json_body(
+            api_request,
+            file_hash,
+            attempt_no
         )
-        adjustments = api_request.adjustments
-        original_image_size_in_bytes = api_request.one_pass_info.file_size
-
-        if not force_reprocessing:
-            force_reprocessing = False
-        if not adjustments:
-            adjustments = {}
-
-        expected_optimized_image_filename = (
-            use_envelope_options.expected_optimized_image_filename
-            if use_envelope_options else None
-        )
-        if not expected_optimized_image_filename:
-            expected_optimized_image_filename = (
-                "/python-sdk-call/at-hash/optimized/" + file_hash
-            )
-
-        # Let's use a codified relative path so that we can test-match in tests
-        use_relative_path = (
-            "/python-sdk-call/at-hash/input/" + file_hash + "/attempt_" +
-            str(attempt_no)
-        )
-
-        request_json_body = {
-            "image_file_hash": file_hash,
-            "original_image_size_in_bytes": original_image_size_in_bytes,
-            "relative_path": use_relative_path,
-            "expected_optimized_image_filename":
-                expected_optimized_image_filename,
-            "precursor_name": api_request.precursor_name.value,
-            "domains": [
-                self._domain
-            ],
-            "force_reprocessing": force_reprocessing,
-            "shimmercat_job_record": {
-                "adjustments": adjustments,
-            }
-        }
-        max_overhead = (
-            use_envelope_options.max_overhead if use_envelope_options else None
-        )
-        if max_overhead:
-            request_json_body["max_overhead"] = max_overhead
         retry_options = RetryOptions(
             attempts=attempts,
             start_timeout=20,
@@ -322,6 +244,7 @@ class ToilmoreHTTPRequester(object):
                 headers={
                     'Authorization': self._make_auth_token()
                 },
+                ssl=self._sslcontext
             ) as response:
                 status_code = response.status
                 if int(status_code / 100) == 5:
@@ -345,6 +268,65 @@ class ToilmoreHTTPRequester(object):
                         RequestIssuesEnum.BAD_JSON
                     )
 
+    def _get_request_json_body(
+        self,
+        api_request: ApiRequest,
+        file_hash: str,
+        attempt_no: int
+    ) -> dict:
+
+        use_envelope_options = api_request.envelope_options
+        force_reprocessing = (
+            use_envelope_options.force_reprocessing
+            if use_envelope_options else False
+        )
+        adjustments = api_request.adjustments
+        original_image_size_in_bytes = api_request.one_pass_info.file_size
+
+        if not force_reprocessing:
+            force_reprocessing = False
+        if not adjustments:
+            adjustments = {}
+
+        expected_optimized_image_filename = (
+            use_envelope_options.expected_optimized_image_filename
+            if use_envelope_options else None
+        )
+        if not expected_optimized_image_filename:
+            expected_optimized_image_filename = (
+                f"/python-sdk-call/at-hash/optimized/{file_hash}"
+            )
+        # Let's use a codified relative path so that we can test-match in tests
+        use_relative_path = (
+            f"/python-sdk-call/at-hash/input/{file_hash}/attempt_{attempt_no}"
+        )
+        request_json_body = {
+            "image_file_hash": file_hash,
+            "original_image_size_in_bytes": original_image_size_in_bytes,
+            "relative_path": use_relative_path,
+            "expected_optimized_image_filename":
+                expected_optimized_image_filename,
+            "precursor_name": api_request.precursor_name.value,
+            "domains": [
+                self._domain
+            ],
+            "force_reprocessing": force_reprocessing,
+            "shimmercat_job_record": {
+                "task_thread": file_hash,
+            }
+        }
+        if self._api_endpoint.api_module == constants.LUX_API_KEY:
+            request_json_body['shimmercat_job_record'].update({
+                "adjustments": adjustments
+            })
+        max_overhead = (
+            use_envelope_options.max_overhead if use_envelope_options else None
+        )
+        if max_overhead:
+            request_json_body["max_overhead"] = int(max_overhead)
+
+        return request_json_body
+
     def _make_auth_token(self):
         return "Token {} Domain {}".format(
             self._api_token,
@@ -364,14 +346,33 @@ class ToilmoreHTTPRequester(object):
 
     async def request_presigned(self, presigned_url: str) -> Callable:
         try:
-            response = await self._http_client_session.get(
-                presigned_url
+            retry_options = RetryOptions(
+                attempts=RETRY_COUNT,
+                start_timeout=10,
+                max_timeout=180,
+                statuses=[500, ]
             )
-            status_code = response.status
-            if int(status_code / 100) == 5:
-                raise RequestIssueException(RequestIssuesEnum.FIVE_HUNDRED)
-            response_content = await response.content.read()
-            return await stream_from_argument(response_content)
+            retry_client = RetryClient(
+                raise_for_status=False,
+                retry_options=retry_options,
+                logger=self._logger if self._debug_mode else None
+            )
+            retry_client._client = self._http_client_session
+            async with retry_client.get(
+                presigned_url,
+                ssl=self._sslcontext
+            ) as response:
+                status_code = response.status
+                response_content = await response.content.read()
+                if int(status_code / 100) == 5:
+                    self._debug_message(
+                        UseTag.DOWNLOAD_FROM_PRESIGNED_URL,
+                        "HTTP 50x Error: {}".format(response_content)
+                    )
+                    raise RequestIssueException(
+                        RequestIssuesEnum.FIVE_HUNDRED
+                    )
+                return await stream_from_argument(response_content)
         except aiohttp.ClientConnectionError:
             raise RequestIssueException(RequestIssuesEnum.CONNECTION_ERROR)
         except aiohttp.ServerTimeoutError:
