@@ -240,7 +240,7 @@ class Toilmore(object):
         :param image_arg: The contents of the image to optimize.
         :param precursor: The precursor (see above).
         :param adjustments: see Adjustments: https://pixellena.com/lux-api/
-     *                    in our API docs.
+*           in our API docs.
         :param envelope_options: Object with other misc options that the API
             accept, optional.
         :return:
@@ -252,7 +252,6 @@ class Toilmore(object):
         stream_factory = await stream_from_argument(image_arg)
         st = stream_factory()
         one_pass_info = await pass_and_compute_hash(st)
-        print(one_pass_info)
         # This is not an accepted image input...
         if not one_pass_info.image_type:
             rn = RejectionNotice(
@@ -314,7 +313,7 @@ class Toilmore(object):
 
         if success_or_deny == 'processing':
             self.emit_debug_message('Image is processing yet...')
-            return await self._wait_for_ready(0, api_request, 0)
+            return await self._wait_for_ready(0, api_request, 0, image_arg)
 
         # If came here, it's probably because we need a file.
         if res_status == 404 and success_or_deny == 'expecting-file':
@@ -407,7 +406,8 @@ class Toilmore(object):
                 return await self._wait_for_ready(
                     0,
                     api_request,
-                    0
+                    0,
+                    image_arg
                 )
             else:
                 self.emit_debug_message(
@@ -454,7 +454,8 @@ class Toilmore(object):
         self,
         total_time_used: int,
         api_request: ApiRequest,
-        attempt_no: int
+        attempt_no: int,
+        image_arg: ImageArg
     ) -> OptimizationResponse:
         # 20 minutes is kind of long and should not happen, but ...
         # Make this configurable.
@@ -496,11 +497,23 @@ class Toilmore(object):
                 again = self._wait_for_ready(
                     30 + total_time_used,
                     api_request,
-                    attempt_no + 1
+                    attempt_no + 1,
+                    image_arg
                 )
 
                 return await again
 
+            # If came here, it's probably because we need a file.
+            if success_or_deny == 'expecting-file':
+                self.emit_debug_message(
+                    '_wait_for_ready: "expecting-file", re-uploading the file...'
+                )
+                stream_factory = await stream_from_argument(image_arg)
+                return await self._continue_to_submit(
+                    stream_factory,
+                    api_request
+                )
+
             # Working around infelicity with temporary errors.
             if res0.status == 404:
                 # This may happen after submit in the time where the trace
@@ -516,7 +529,8 @@ class Toilmore(object):
                 again = self._wait_for_ready(
                     30 + total_time_used,
                     api_request,
-                    attempt_no + 1
+                    attempt_no + 1,
+                    image_arg
                 )
 
                 return await again
